#include <Wire.h>
#include <LiquidCrystal_I2C.h>

// Konstanten
const int fanPin = 3;                 // PWM Pin des Lüfters
const int fanPower = 4;               // Pin zum schalten des Transistors für Lüfter An/Aus
const int ntc = A0;                   // Pin für den 10kO NTC Wiederstand
const long interval = 1000;           // Interval wie oft die Temperatur abgefragt wird (milliseunden)
const int abfrageZahl = 5;            // Je mehr abfragen, desto stabiler isr das Ergebnis, dauert aber länger
const int ntcNominal = 10000;         // Wiederstand des NTC bei Nominaltemperatur
const int tempNominal = 25;           // Temperatur bei der der NTC den angegebenen Wiederstand hat
const int bCoefficient = 3950;        // Beta Coefficient(B25 aus Datenblatt des NTC)
const int serienWiederstand = 10000;  // Wert des Wiederstands der mit dem NTC in Serie geschalten ist
const int tMin = 22;                  // Minimaltemperatur. Unterschalb schaltet der Lüfter ab
const int tMax = 30;                  // Maximaltemperatur. Ab dieser Temperatur läuft der Lüfter mit 100%

// Variablen
int fanSpeed = 0;                // Variable für den PWM Wert des Lüfters
boolean fanOut = true;           // Zustand des Lüfters (An/Aus)
int abfrage[abfrageZahl];        // Array Variable für das Mitteln der Temperatur
float durchschnitt = 0;          // Variable für das Mitteln der Temperatur
float temp;                      // Variable für die Berechnung der temperatur nach Steinhart
float minTemp = 0;               // Variable für die niedrigste gemessene Temperatur
float maxTemp = 0;               // Variable für die höchste gemessene Temperatur


// Setzt die I2C Adresse des LCD auf 0x27 für ein 20 Zeichen und 4 Zeilen Display
LiquidCrystal_I2C lcd(0x27, 20, 4);


void setup() {
  Serial.begin(9600);
  pinMode(ntc, INPUT);        // Setzt den Pin des NTC Wiederstands als Eingang
  pinMode(fanPin, OUTPUT);    // Setzt den PWM Pin des Lüfters als Ausgang
  pinMode(fanPower, OUTPUT);  // Setzt den Power Pin des Lüfters als Ausgang
  
  lcd.begin();                // Initialisiert das LCD
  lcd.backlight();            // LCD Hintergrundbeleuchtung einschalten
  lcd.print("Hallo Lukas");   // Begrüßungsnachricht anzeigen
  delay(2000);                // Zeige die Begrüßung 2s an
  lcd.clear();                // Display leeren

// Erste Temperatuerfassung um die min/max Werte festzulegen
  temperaturberechnung();     
  minTemp = temp;
  maxTemp = temp;
}


void loop()
{
  // Startet die Temperaturerfassungsroutine
  temperaturberechnung();
 
  // Wenn die Temperatur kleiner als die minTemp ist, den Wert als neue minTemp speichern
  if (temp < minTemp) minTemp = temp;
  // Wenn die Temperatur größer als die maxTemp ist, den Wert als neue maxTemp speichern
  if (temp > maxTemp) maxTemp = temp;

  // Wenn die Temperatur unter die von tMinn fällt, schält der Lüfter ab
  if (temp < tMin) 
  {
    fanOut = true;
    digitalWrite(fanPower, LOW);
    fanSpeed = 0;
  }
  
  
  // Hysterese(Einschaltverzögerung)
  if(temp >= (tMin + 1))
  {
    fanOut = false;
    digitalWrite(fanPower, HIGH);
  }
  
  // Lüftergeschwindigkeit über den Temperaturbereich einstellen
  // TMin->0% PWM | TMax->100% PWM
  
  if (fanOut == false)
  {
    fanSpeed = map(temp, tMin, tMax, 100, 255);
    Serial.print("fanSpeed: ");
    Serial.println(fanSpeed);
  }

  analogWrite(fanPin, fanSpeed);
  
  // Ausgabe der aktuellen Temperatur in Zeile 1
  lcd.setCursor(0, 0);
  lcd.print("Mom. Temp ");
  lcd.print(temp);
  lcd.print(" *C");

// Ausgabe der niedrigsten Temperatur in Zeile 2
  lcd.setCursor(0, 1);
  lcd.print("Min. Temp ");
  lcd.print(minTemp);
  lcd.print(" *C");

  // Ausgabe der höchsten Temperatur in Zeile 3
  lcd.setCursor(0, 2);
  lcd.print("Max. Temp ");
  lcd.print(maxTemp);
  lcd.print(" *C");

  delay(1000);
  Serial.print("fanOunt: ");
  Serial.println(fanOut);
} 


void temperaturberechnung()
{
  // Nimmt N Abfragen in einer Reihe, mit einem kurzen delay
  for (int i=0; i < abfrageZahl; i++)
  {
    abfrage[i] = analogRead(ntc);
    delay(10);
  }
  
  // Mittelt alle Abfragen
  durchschnitt = 0;
  for (int i=0; i < abfrageZahl; i++)
  {
    durchschnitt += abfrage[i];
  }
  durchschnitt /= abfrageZahl;
  
  // Umwandlung des Wertes in Wiederstand
  durchschnitt = 1023 / durchschnitt - 1;
  durchschnitt = serienWiederstand / durchschnitt;
  
  // Umrechnung aller Ergebnisse in die Temperatur mittels einer Steinhard Berechnung
  temp = durchschnitt / ntcNominal;     // (R/Ro)
  temp = log(temp);                     // ln(R/Ro)
  temp /= bCoefficient;                 // 1/B * ln(R/Ro)
  temp += 1.0 / (tempNominal + 273.15); // + (1/To)
  temp = 1.0 / temp;                    // Invertieren
  temp -= 273.15;                       // Umwandeln in °C
}
